class com.cintegrity.Lib.Collection.SortedSet
    implements com.cintegrity.Lib.Collection.Collection
    :
  define private temp-table ctt_SortedSet
      field in_Sequence as integer
      field ob_AnObject as class Progress.Lang.Object
      index by_Sequence as primary unique in_Sequence
      index by_Object ob_AnObject.
  define private variable cin_CurrentSequence as integer no-undo initial 0.
  define private variable cin_HighestSequence as integer no-undo initial 0.

  destructor public SortedSet():
    empty temp-table ctt_SortedSet.
  end destructor.
  /* ---------------------------------------- Implement Collection interface */
  method public integer size():
    return cin_HighestSequence.
  end method.

  method public logical isEmpty():
    return (cin_HighestSequence eq 0).
  end method.

  method public logical containsElement(
      input mob_Element as class Progress.Lang.Object
      ):
    find ctt_SortedSet where mob_Element eq ctt_SortedSet.ob_AnObject no-error.
    return available( ctt_SortedSet ).
  end method.

  method public logical addElement(
      input mob_Element as class Progress.Lang.Object
      ):
                                                        /* Ensure uniqueness */
    if not containsElement( mob_Element )
    then do:
      create ctt_SortedSet.
      assign
          cin_HighestSequence = cin_HighestSequence + 1
          ctt_SortedSet.in_Sequence = cin_HighestSequence
          ctt_SortedSet.ob_AnObject = mob_Element
          cin_CurrentSequence = cin_HighestSequence
          .
      return true.
    end.
    else return false.
  end method.

  method public logical removeElement(
      input mob_Element as class Progress.Lang.Object
      ):
    define variable min_DeletedElementSeq as integer no-undo.
    find ctt_SortedSet where mob_Element eq ctt_SortedSet.ob_AnObject no-error.
    if available( ctt_SortedSet )
    then do:
      min_DeletedElementSeq = ctt_SortedSet.in_Sequence.
      delete ctt_SortedSet.
      if min_DeletedElementSeq LT cin_HighestSequence
      then do:
        for each ctt_SortedSet
            where ctt_SortedSet.in_Sequence GE min_DeletedElementSeq
            :
          ctt_SortedSet.in_Sequence = ctt_SortedSet.in_Sequence - 1.
        end.
        cin_HighestSequence = ctt_SortedSet.in_Sequence.
        cin_CurrentSequence = min_DeletedElementSeq + 1.
      end.
      else do:
        cin_HighestSequence = cin_HighestSequence - 1.
        cin_CurrentSequence = cin_HighestSequence.
      end.
      return true.
    end.
    else return false.
  end method.

  /* -------------------------------------------------- Implementation deferred
  method public logical containsAll(
      input mob_Collection as class com.cintegrity.Lib.Collection.Collection 
      ):
  end method.

  method public void addAll(
      input mob_Collection as class com.cintegrity.Lib.Collection.Collection
      ):
  end method.

  method public void removeAll(
      input mob_Collection as class com.cintegrity.Lib.Collection.Collection
      ):
  end method.

  method public void retainAll(
      input mob_Collection as class com.cintegrity.Lib.Collection.Collection
      ):
  end method.
  -------------------------------------------------------------------------- */

  method public void clearAll():
    empty temp-table ctt_SortedSet.
    cin_CurrentSequence = 0.
    cin_HighestSequence = 0.
  end method.

  method public logical hasNextElement():
    return can-find( first ctt_SortedSet where ctt_SortedSet.in_Sequence GT cin_CurrentSequence ).
  end method.

  method public logical hasPrevElement():
    return can-find( first ctt_SortedSet where ctt_SortedSet.in_Sequence LT cin_CurrentSequence ).
  end method.

  method public class Progress.Lang.Object nextElement( ):
    find ctt_SortedSet
        where ctt_SortedSet.in_Sequence = ( cin_CurrentSequence + 1 )
        no-error.
    if available( ctt_SortedSet )
    then do:
      cin_CurrentSequence = ctt_SortedSet.in_Sequence.
      return ctt_SortedSet.ob_AnObject.
    end.
    else do:
      /* raise "No Such Element" exception */
      return ?.
    end.
  end method.

  method public class Progress.Lang.Object prevElement( ):
    find ctt_SortedSet
        where ctt_SortedSet.in_Sequence = ( cin_CurrentSequence - 1 )
        no-error.
    if available( ctt_SortedSet )
    then do:
      cin_CurrentSequence = ctt_SortedSet.in_Sequence.
      return ctt_SortedSet.ob_AnObject.
    end.
    else do:
      /* raise "No Such Element" exception */
      return ?.
    end.
  end method.

  method public class Progress.Lang.Object setElement(
      input mob_Element as class Progress.Lang.Object
      ):
    define variable mob_OldObject as class Progress.Lang.Object no-undo.

    find ctt_SortedSet
        where ctt_SortedSet.in_Sequence = cin_CurrentSequence
        no-error.
    if available( ctt_SortedSet )
    then do:
      mob_OldObject = ctt_SortedSet.ob_AnObject.
      delete ctt_SortedSet.
      create ctt_SortedSet.
      assign
          ctt_SortedSet.in_Sequence = cin_CurrentSequence
          ctt_SortedSet.ob_AnObject = mob_Element
          .
      return mob_OldObject.
    end.
    else do:
      /* raise "No Such Element" exception */
      return ?.
    end.
  end method.
  /* ------------------------------------------- End of Collection interface */

  /* ---------------------------------------------- Extensions for SortedSet */
  method public class Progress.Lang.Object getFirst():
    cin_CurrentSequence = 1.
    find ctt_SortedSet
        where cin_CurrentSequence eq ctt_SortedSet.in_Sequence
        no-error.
    if available( ctt_SortedSet )
    then return ctt_SortedSet.ob_AnObject.
    else do:
      /* raise "No Such Element" exception */
      return ?.
    end.
  end method.

  method public class Progress.Lang.Object getLast():
    cin_CurrentSequence = cin_HighestSequence.
    find ctt_SortedSet
        where cin_CurrentSequence = ctt_SortedSet.in_Sequence
        no-error.
    if available( ctt_SortedSet )
    then return ctt_SortedSet.ob_AnObject.
    else do:
      /* raise "No Such Element" exception */
      return ?.
    end.
  end method.

  /* -------------------------------------------------- Implementation deferred
  method public class com.cintegrity.Lib.Collection.SortedSet subSet(
      input mob_MinObject as class Progress.Lang.Object,
      input mob_MaxObject as class Progress.Lang.Object
      ):

  end method.

  method public class com.cintegrity.Lib.Collection.SortedSet headSet(
      input mob_MaxObject as class Progress.Lang.Object
      ):

  end method.

  method public class com.cintegrity.Lib.Collection.SortedSet tailSet(
      input mob_MinObject as class Progress.Lang.Object
      ):

  end method.

  method public class com.cintegrity.Lib.Collection.SortedSet CopySubSet(
      input mob_MinObject as class Progress.Lang.Object
      input mob_MaxObject as class Progress.Lang.Object
      ):

  end method.

  method public class com.cintegrity.Lib.Collection.SortedSet copyHeadSet(
      input mob_MaxObject as class Progress.Lang.Object
      ):

  end method.

  method public class com.cintegrity.Lib.Collection.SortedSet copyTailSet(
      input mob_MinObject as class Progress.Lang.Object
      ):

  end method.
  -------------------------------------------------------------------------- */

  method public override class Progress.Lang.Object clone():
    define variable mob_CloneSortedSet as com.cintegrity.Lib.Collection.SortedSet.
    define variable mlg_IsOk as logical no-undo.

    mob_CloneSortedSet = new com.cintegrity.Lib.Collection.SortedSet().
    for each ctt_SortedSet:
      mlg_IsOK = mob_CloneSortedSet:addElement( ctt_SortedSet.ob_AnObject ).
      if not mlg_IsOK
      then do:
                                           /* raise unsuccessful clone error */
        return ?.
      end.
    end.
    return mob_CloneSortedSet.

  end method.

end class.
